----------------------------------------------------------------------
-- This file is ment to provide a layer of abstraction to buttons	--
-- FlexBar2 buttons do not use Action IDs, most code in this file	--
-- is aimed to emulate the behaviour of Action IDs.					--
----------------------------------------------------------------------

local Action =  FlexBar2:NewModule("Action");
Action.Version = 1;

Action.SpellIDCache = {};
local SpellIDCache = Action.SpellIDCache;
function Action:OnEnable()
	self:BuildSpellIDCache();
    self:BuildInventoryCache();
end

function Action:BuildInventoryCache()
end

function Action:BuildSpellIDCache()
	local SpellID = 1;
	while(true) do
		local SpellName, SpellRank = GetSpellName(SpellID, BOOKTYPE_SPELL);
		if(type(SpellName) ~= "string") then break; end
		if(type(SpellRank) == "string" and string.find(SpellRank, "^Rank %d+$")) then
			SpellName = SpellName .. "(" .. SpellRank .. ")";
		end
		SpellIDCache[SpellName] = SpellID;
		SpellID = SpellID + 1;
	end
end

Action.ButtonMixin = FlexBar2.OO.Mixin
{
    "LockAction",
    "UnlockAction",
	"ClearAction",
    "PickupAction",
	"PlaceAction",
	"GetActionTexture",
	"UpdateActionTexture",
    "UpdateCooldownSpinner",
};

function Action.ButtonMixin:Activate()
    self.Frame:RegisterForDrag("LeftButton");
	self:RegisterScript("OnAttributeChanged", function(self, Trigger, Button, AttributeName, AttributeValue) if(AttributeName == "state-parent") then FlexBar2:Debug(type(AttributeValue)); self:UpdateActionTexture(); end end);
	self:RegisterEvent("MODIFIER_STATE_CHANGED", "UpdateActionTexture");
	self:UpdateActionTexture();
end

function Action.ButtonMixin:SetDefaults()
    self.InfoTable.LockAction = false;
	self:UpdateActionTexture();
end

function Action.ButtonMixin:LoadSettings()
    if(self.InfoTable.LockAction == true) then
        self:LockAction();
    else
        self:UnlockAction();
    end
end

function Action.ButtonMixin:LockAction()
    self:UnregisterScript("OnDragStart", "PickupAction");
    self:UnregisterScript("OnReceiveDrag", "PlaceAction");
end

function Action.ButtonMixin:UnlockAction()
    self:RegisterScript("OnDragStart", "PickupAction");
    self:RegisterScript("OnReceiveDrag", "PlaceAction");
end

function Action.ButtonMixin:GetActionTexture()
	local ActionType = self:GetModifiedAttribute("type");
	if(ActionType == "spell") then
		return GetSpellTexture(self:GetModifiedAttribute("spell")) or "";
	elseif(ActionType == "item") then
		local ItemName, ItemLink, ItemRarity, ItemLevel, ItemMinLevel, ItemType, ItemSubType, ItemStackCount, ItemEquipLoc, ItemTexture = GetItemInfo(self:GetModifiedAttribute("item"));
		return ItemTexture or "";
	elseif(ActionType == "macro") then
		local MacroName, MacroTexture, MacroBody, MacroLocal = GetMacroInfo(self:GetModifiedAttribute("macro"));
		return MacroTexture or "";
	else
		return "";
	end
end

function Action.ButtonMixin:UpdateActionTexture()
	FlexBar2:Debug("Updating action texture");
	self:SetIcon(self:GetActionTexture());
end

function Action.ButtonMixin:UpdateCooldownSpinner()
    
end

function Action.ButtonMixin:ClearAction()
	self:SetModifiedAttribute("type", nil);
	self:SetModifiedAttribute("spell", nil);
	self:SetModifiedAttribute("macro", nil);
	self:SetModifiedAttribute("item", nil);
	self:UpdateActionTexture();
end

function Action.ButtonMixin:PickupAction()
	if(InCombatLockdown() == nil) then
		local ActionType = self:GetModifiedAttribute("type");
		FlexBar2:Debug(ActionType);
		if(ActionType == "spell") then
			PickupSpell(SpellIDCache[self:GetModifiedAttribute("spell")], BOOKTYPE_SPELL);
		elseif(ActionType == "item") then
			FlexBar2:Print("Unsupported yet due to blizz weirdness");
		elseif(ActionType == "macro") then
			PickupMacro(self:GetModifiedAttribute("macro"));
		else
			ClearCursor();
		end
		self:ClearAction();
	end
end

function Action.ButtonMixin:PlaceAction()
    if(InCombatLockdown() == nil) then
        local ActionType, ActionId, ActionBook = GetCursorInfo()
		FlexBar2:Debug(ActionType, ActionId, ActionBook);
        if(ActionType ~= nil) then self:PickupAction(); end
        if(ActionType == "spell") then
            self:SetModifiedAttribute("type", "spell");
            local SpellName, SpellRank = GetSpellName(ActionId, ActionBook);
            if(type(SpellRank) == "string" and string.find(SpellRank, "^Rank %d+$")) then
                SpellName = SpellName .. "(" .. SpellRank .. ")";
            end
            self:SetModifiedAttribute("spell", SpellName);
        elseif(ActionType == "item") then
            local ItemName, ItemLink, ItemRarity, ItemLevel, ItemMinLevel, ItemType, ItemSubType, ItemStackCount, ItemEquipLoc, ItemTexture = GetItemInfo(ActionId);
			self:SetModifiedAttribute("type", "item");
			self:SetModifiedAttribute("item", ItemName);
            self:SetModifiedAttribute("BagInfo", {"", ""});
        elseif(ActionType == "macro") then
			local MacroName, MacroTexture, MacroBody, MacroLocal = GetMacroInfo(ActionId);
			self:SetModifiedAttribute("type", "macro");
			self:SetModifiedAttribute("macro", ActionId);
        end
		self:UpdateActionTexture();
    else
        FlexBar2:Print("Actions cannot be changed in combat, sorry");
    end
end
